@@ -65,7 +65,7 @@ enum {
 	LAST_SIGNAL
 };
 
-static guint e_book_signals [LAST_SIGNAL];
+static guint e_book_signals[LAST_SIGNAL];
 
 struct _EBookPrivate {
 	EGdbusBook *gdbus_book;
@@ -80,6 +80,7 @@ struct _EBookPrivate {
 	gboolean cap_queried;
 };
 
+static guint active_books = 0, book_connection_closed_id = 0;
 static EGdbusBookFactory *book_factory_proxy = NULL;
 static GStaticRecMutex book_factory_proxy_lock = G_STATIC_REC_MUTEX_INIT;
 #define LOCK_FACTORY()   g_static_rec_mutex_lock (&book_factory_proxy_lock)
@@ -135,7 +136,7 @@ gdbus_book_closed_cb (GDBusConnection *connection, gboolean remote_peer_vanished
 
 	UNLOCK_FACTORY ();
 
-	g_signal_emit (G_OBJECT (book), e_book_signals [BACKEND_DIED], 0);
+	g_signal_emit (G_OBJECT (book), e_book_signals[BACKEND_DIED], 0);
 }
 
 static void
@@ -187,6 +188,10 @@ e_book_finalize (GObject *object)
 
 	if (G_OBJECT_CLASS (e_book_parent_class)->finalize)
 		G_OBJECT_CLASS (e_book_parent_class)->finalize (object);
+
+	LOCK_FACTORY ();
+	active_books--;
+	UNLOCK_FACTORY ();
 }
 
 static void
@@ -194,7 +199,7 @@ e_book_class_init (EBookClass *e_book_class)
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS (e_book_class);
 
-	e_book_signals [WRITABLE_STATUS] =
+	e_book_signals[WRITABLE_STATUS] =
 		g_signal_new ("writable_status",
 			      G_OBJECT_CLASS_TYPE (gobject_class),
 			      G_SIGNAL_RUN_LAST,
@@ -204,7 +209,7 @@ e_book_class_init (EBookClass *e_book_class)
 			      G_TYPE_NONE, 1,
 			      G_TYPE_BOOLEAN);
 
-	e_book_signals [CONNECTION_STATUS] =
+	e_book_signals[CONNECTION_STATUS] =
 		g_signal_new ("connection_status",
 			      G_OBJECT_CLASS_TYPE (gobject_class),
 			      G_SIGNAL_RUN_LAST,
@@ -214,7 +219,7 @@ e_book_class_init (EBookClass *e_book_class)
 			      G_TYPE_NONE, 1,
 			      G_TYPE_BOOLEAN);
 
-	e_book_signals [AUTH_REQUIRED] =
+	e_book_signals[AUTH_REQUIRED] =
 		g_signal_new ("auth_required",
 			      G_OBJECT_CLASS_TYPE (gobject_class),
 			      G_SIGNAL_RUN_LAST,
@@ -223,7 +228,7 @@ e_book_class_init (EBookClass *e_book_class)
 			      e_book_marshal_NONE__NONE,
 			      G_TYPE_NONE, 0);
 
-	e_book_signals [BACKEND_DIED] =
+	e_book_signals[BACKEND_DIED] =
 		g_signal_new ("backend_died",
 			      G_OBJECT_CLASS_TYPE (gobject_class),
 			      G_SIGNAL_RUN_LAST,
@@ -243,6 +248,10 @@ e_book_init (EBook *book)
 {
 	EBookPrivate *priv = E_BOOK_GET_PRIVATE (book);
 
+	LOCK_FACTORY ();
+	active_books++;
+	UNLOCK_FACTORY ();
+
 	priv->gdbus_book = NULL;
 	priv->source = NULL;
 	priv->uri = NULL;
@@ -260,6 +269,13 @@ book_factory_proxy_closed_cb (GDBusConnection *connection, gboolean remote_peer_
 	GError *err = NULL;
 
 	LOCK_FACTORY ();
+
+	if (book_connection_closed_id) {
+		g_dbus_connection_signal_unsubscribe (connection, book_connection_closed_id);
+		book_connection_closed_id = 0;
+		g_signal_handlers_disconnect_by_func (connection, book_factory_proxy_closed_cb, NULL);
+	}
+
 	if (book_factory_proxy) {
 		g_object_unref (book_factory_proxy);
 		book_factory_proxy = NULL;
@@ -271,7 +287,7 @@ book_factory_proxy_closed_cb (GDBusConnection *connection, gboolean remote_peer_
 	if (err) {
 		g_debug ("GDBus connection is closed%s: %s", remote_peer_vanished ? ", remote peer vanished" : "", err->message);
 		g_error_free (err);
-	} else {
+	} else if (active_books) {
 		g_debug ("GDBus connection is closed%s", remote_peer_vanished ? ", remote peer vanished" : "");
 	}
 
@@ -313,7 +329,7 @@ e_book_activate (GError **error)
 
 	connection = g_dbus_proxy_get_connection (G_DBUS_PROXY (book_factory_proxy));
 	g_dbus_connection_set_exit_on_close (connection, FALSE);
-	g_dbus_connection_signal_subscribe (connection,
+	book_connection_closed_id = g_dbus_connection_signal_subscribe (connection,
 		NULL,						/* sender */
 		"org.freedesktop.DBus",				/* interface */
 		"NameOwnerChanged",				/* member */
@@ -336,7 +352,7 @@ writable_cb (EGdbusBook *object, gboolean writable, EBook *book)
 
 	book->priv->writable = writable;
 
-	g_signal_emit (G_OBJECT (book), e_book_signals [WRITABLE_STATUS], 0, writable);
+	g_signal_emit (G_OBJECT (book), e_book_signals[WRITABLE_STATUS], 0, writable);
 }
 
 static void
@@ -346,7 +362,7 @@ connection_cb (EGdbusBook *object, gboolean connected, EBook *book)
 
 	book->priv->connected = connected;
 
-	g_signal_emit (G_OBJECT (book), e_book_signals [CONNECTION_STATUS], 0, connected);
+	g_signal_emit (G_OBJECT (book), e_book_signals[CONNECTION_STATUS], 0, connected);
 }
 
 static void
@@ -354,7 +370,7 @@ auth_required_cb (EGdbusBook *object, EBook *book)
 {
 	g_return_if_fail (E_IS_BOOK (book));
 
-	g_signal_emit (G_OBJECT (book), e_book_signals [AUTH_REQUIRED], 0);
+	g_signal_emit (G_OBJECT (book), e_book_signals[AUTH_REQUIRED], 0);
 }
 
 /**
