@@ -24,7 +24,7 @@
 
 #include <string.h>
 #include <glib/gi18n-lib.h>
-#include "libedataserver/e-data-server-util.h"
+#include <libedataserver/e-data-server-util.h>
 #include <libecal/e-cal-time-util.h>
 
 #include "e-cal-backend-sexp.h"
@@ -1189,7 +1189,7 @@ entry_compare(SearchContext *ctx, struct _ESExp *f,
 		propname = argv[0]->value.string;
 
 		any_field = !strcmp(propname, "x-evolution-any-field");
-		for (i = 0; i < G_N_ELEMENTS (prop_info_table); i ++) {
+		for (i = 0; i < G_N_ELEMENTS (prop_info_table); i++) {
 			if (any_field
 			    || !strcmp (prop_info_table[i].query_prop, propname)) {
 				info = &prop_info_table[i];
@@ -1272,6 +1272,26 @@ static struct {
 	{ "percent-complete?", func_percent_complete, 0 }
 };
 
+/**
+ * e_cal_backend_sexp_evaluate_occur_times:
+ * @sexp: An #ESExp object.
+ * @start: Start of the time window will be stored here.
+ * @end: End of the time window will be stored here.
+ *
+ * Determines biggest time window given by expressions "occur-in-range" in sexp.
+ *
+ * Since: 2.32
+ */
+gboolean
+e_cal_backend_sexp_evaluate_occur_times(ECalBackendSExp *sexp, time_t *start, time_t *end)
+{
+	g_return_val_if_fail (sexp != NULL, FALSE);
+	g_return_val_if_fail (start != NULL, FALSE);
+	g_return_val_if_fail (end != NULL, FALSE);
+
+	return e_sexp_evaluate_occur_times (sexp->priv->search_sexp, start, end);
+}
+
 /**
  * e_cal_backend_sexp_match_comp:
  * @sexp: An #ESExp object.
@@ -1446,3 +1466,50 @@ e_cal_backend_sexp_init (ECalBackendSExp *sexp)
 	sexp->priv = priv;
 	priv->search_context = g_new (SearchContext, 1);
 }
+#ifdef TESTER
+static void
+test_query (const gchar * query)
+{
+	ECalBackendSExp *sexp = e_cal_backend_sexp_new (query);
+	time_t start, end;
+
+	gboolean generator = e_cal_backend_sexp_evaluate_occur_times(sexp, &start, &end);
+
+	if (generator) {
+		printf ("%s: %ld - %ld\n", query, start, end);
+	} else {
+		printf ("%s: no time prunning possible\n", query);
+	}
+}
+
+gint main(gint argc, gchar **argv)
+{
+	g_type_init();
+
+	/* e_sexp_add_variable(f, 0, "test", NULL); */
+
+	if (argc < 2 || !argv[1])
+	{
+		test_query ("(occur-in-time-range? (make-time \"20080727T220000Z\") (make-time \"20080907T220000Z\"))");
+		test_query ("(due-in-time-range? (make-time \"20080727T220000Z\") (make-time \"20080907T220000Z\"))");
+		test_query ("(has-alarms-in-range? (make-time \"20080727T220000Z\") (make-time \"20080907T220000Z\"))");
+		test_query ("(completed-before? (make-time \"20080727T220000Z\") )");
+
+		test_query ("(and (occur-in-time-range? (make-time \"20080727T220000Z\") (make-time \"20080907T220000Z\")) #t)");
+		test_query ("(or (occur-in-time-range? (make-time \"20080727T220000Z\")(make-time \"20080907T220000Z\")) #t)");
+
+		test_query ("(and (contains? \"substring\") (has-categories? \"blah\"))");
+		test_query ("(or (occur-in-time-range? (make-time \"20080727T220000Z\") (make-time \"20080907T220000Z\")) (contains? \"substring\"))");
+
+		test_query ("(and (and (occur-in-time-range? (make-time \"20080727T220000Z\") (make-time \"20080907T220000Z\"))"
+			    " (or (contains? \"substring\") (has-categories? \"blah\"))) (has-alarms?))");
+
+		test_query ("(or (and (occur-in-time-range? (make-time \"20080727T220000Z\") (make-time \"20080907T220000Z\"))"
+			    " (or (contains? \"substring\") (has-categories? \"blah\"))) (has-alarms?))");
+	}
+	else
+		test_query(argv[1]);
+
+	return 0;
+}
+#endif
