@@ -30,7 +30,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "camel-exception.h"
 #include "camel-http-stream.h"
 #include "camel-mime-utils.h"
 #include "camel-net-utils.h"
@@ -39,148 +38,35 @@
 #include "camel-stream-buffer.h"
 #include "camel-tcp-stream-raw.h"
 
-#ifdef HAVE_SSL
+#ifdef CAMEL_HAVE_SSL
 #include "camel-tcp-stream-ssl.h"
 #endif
 
+#define SSL_FLAGS (CAMEL_TCP_STREAM_SSL_ENABLE_SSL2 | CAMEL_TCP_STREAM_SSL_ENABLE_SSL3)
+
+#ifdef G_OS_WIN32
+#include <winsock2.h>
+#include <ws2tcpip.h>
+#endif
+
 #define d(x)
 
-static CamelStreamClass *parent_class = NULL;
-
-static gssize stream_read (CamelStream *stream, gchar *buffer, gsize n);
-static gssize stream_write (CamelStream *stream, const gchar *buffer, gsize n);
-static gint stream_flush  (CamelStream *stream);
-static gint stream_close  (CamelStream *stream);
-static gint stream_reset  (CamelStream *stream);
-
-static void
-camel_http_stream_class_init (CamelHttpStreamClass *camel_http_stream_class)
-{
-	CamelStreamClass *camel_stream_class =
-		CAMEL_STREAM_CLASS (camel_http_stream_class);
-
-	parent_class = CAMEL_STREAM_CLASS (camel_type_get_global_classfuncs (camel_stream_get_type ()));
-
-	/* virtual method overload */
-	camel_stream_class->read = stream_read;
-	camel_stream_class->write = stream_write;
-	camel_stream_class->flush = stream_flush;
-	camel_stream_class->close = stream_close;
-	camel_stream_class->reset = stream_reset;
-}
-
-static void
-camel_http_stream_init (gpointer object, gpointer klass)
-{
-	CamelHttpStream *http = CAMEL_HTTP_STREAM (object);
-
-	http->parser = NULL;
-	http->content_type = NULL;
-	http->headers = NULL;
-	http->session = NULL;
-	http->url = NULL;
-	http->proxy = NULL;
-	http->authrealm = NULL;
-	http->authpass = NULL;
-	http->statuscode = 0;
-	http->raw = NULL;
-}
-
-static void
-camel_http_stream_finalize (CamelObject *object)
-{
-	CamelHttpStream *http = CAMEL_HTTP_STREAM (object);
-
-	if (http->parser)
-		camel_object_unref(http->parser);
-
-	if (http->content_type)
-		camel_content_type_unref (http->content_type);
-
-	if (http->headers)
-		camel_header_raw_clear (&http->headers);
-
-	if (http->session)
-		camel_object_unref(http->session);
-
-	if (http->url)
-		camel_url_free (http->url);
-
-	if (http->proxy)
-		camel_url_free (http->proxy);
-
-	g_free (http->authrealm);
-	g_free (http->authpass);
-
-	if (http->raw)
-		camel_object_unref(http->raw);
-	if (http->read)
-		camel_object_unref(http->read);
-}
-
-CamelType
-camel_http_stream_get_type (void)
-{
-	static CamelType type = CAMEL_INVALID_TYPE;
-
-	if (type == CAMEL_INVALID_TYPE) {
-		type = camel_type_register (camel_stream_get_type (),
-					    "CamelHttpStream",
-					    sizeof (CamelHttpStream),
-					    sizeof (CamelHttpStreamClass),
-					    (CamelObjectClassInitFunc) camel_http_stream_class_init,
-					    NULL,
-					    (CamelObjectInitFunc) camel_http_stream_init,
-					    (CamelObjectFinalizeFunc) camel_http_stream_finalize);
-	}
-
-	return type;
-}
-
-/**
- * camel_http_stream_new:
- * @method: HTTP method
- * @session: active session
- * @url: URL to act upon
- *
- * Return value: a http stream
- **/
-CamelStream *
-camel_http_stream_new (CamelHttpMethod method, struct _CamelSession *session, CamelURL *url)
-{
-	CamelHttpStream *stream;
-	gchar *str;
-
-	g_return_val_if_fail(CAMEL_IS_SESSION(session), NULL);
-	g_return_val_if_fail(url != NULL, NULL);
-
-	stream = CAMEL_HTTP_STREAM (camel_object_new (camel_http_stream_get_type ()));
-
-	stream->method = method;
-	stream->session = session;
-	camel_object_ref(session);
-
-	str = camel_url_to_string (url, 0);
-	stream->url = camel_url_new (str, NULL);
-	g_free (str);
-
-	return (CamelStream *)stream;
-}
-
-#define SSL_FLAGS (CAMEL_TCP_STREAM_SSL_ENABLE_SSL2 | CAMEL_TCP_STREAM_SSL_ENABLE_SSL3)
+G_DEFINE_TYPE (CamelHttpStream, camel_http_stream, CAMEL_TYPE_STREAM)
 
 static CamelStream *
-http_connect (CamelHttpStream *http, CamelURL *url)
+http_connect (CamelHttpStream *http,
+              CamelURL *url,
+              GError **error)
 {
+	CamelTcpStream *tcp_stream;
 	CamelStream *stream = NULL;
-	struct addrinfo *ai, hints = { 0 };
 	gint errsave;
 	gchar *serv;
 
 	d(printf("connecting to http stream @ '%s'\n", url->host));
 
 	if (!g_ascii_strcasecmp (url->protocol, "https")) {
-#ifdef HAVE_SSL
+#ifdef CAMEL_HAVE_SSL
 		stream = camel_tcp_stream_ssl_new (http->session, url->host, SSL_FLAGS);
 #endif
 	} else {
@@ -189,6 +75,10 @@ http_connect (CamelHttpStream *http, CamelURL *url)
 
 	if (stream == NULL) {
 		errno = EINVAL;
+		g_set_error (
+			error, G_IO_ERROR,
+			g_io_error_from_errno (errno),
+			"%s", g_strerror (errno));
 		return NULL;
 	}
 
@@ -198,24 +88,16 @@ http_connect (CamelHttpStream *http, CamelURL *url)
 	} else {
 		serv = url->protocol;
 	}
-	hints.ai_socktype = SOCK_STREAM;
 
-	ai = camel_getaddrinfo(url->host, serv, &hints, NULL);
-	if (ai == NULL) {
-		camel_object_unref (stream);
-		return NULL;
-	}
+	tcp_stream = CAMEL_TCP_STREAM (stream);
 
-	if (camel_tcp_stream_connect (CAMEL_TCP_STREAM (stream), ai) == -1) {
+	if (camel_tcp_stream_connect (tcp_stream, url->host, serv, 0, error) == -1) {
 		errsave = errno;
-		camel_object_unref (stream);
-		camel_freeaddrinfo(ai);
+		g_object_unref (stream);
 		errno = errsave;
 		return NULL;
 	}
 
-	camel_freeaddrinfo(ai);
-
 	http->raw = stream;
 	http->read = camel_stream_buffer_new (stream, CAMEL_STREAM_BUFFER_READ);
 
@@ -223,63 +105,115 @@ http_connect (CamelHttpStream *http, CamelURL *url)
 }
 
 static void
-http_disconnect(CamelHttpStream *http)
+http_disconnect (CamelHttpStream *http)
 {
 	if (http->raw) {
-		camel_object_unref(http->raw);
+		g_object_unref (http->raw);
 		http->raw = NULL;
 	}
 
 	if (http->read) {
-		camel_object_unref(http->read);
+		g_object_unref (http->read);
 		http->read = NULL;
 	}
 
 	if (http->parser) {
-		camel_object_unref(http->parser);
+		g_object_unref (http->parser);
 		http->parser = NULL;
 	}
 }
 
-static const gchar *
-http_next_token (const guchar *in)
-{
-	const guchar *inptr = in;
-
-	while (*inptr && !isspace ((gint) *inptr))
-		inptr++;
-
-	while (*inptr && isspace ((gint) *inptr))
-		inptr++;
-
-	return (const gchar *) inptr;
-}
-
 static gint
-http_get_statuscode (CamelHttpStream *http)
+http_method_invoke (CamelHttpStream *http,
+                    GError **error)
 {
-	const gchar *token;
-	gchar buffer[4096];
+	const gchar *method = NULL, *use_url;
+	gchar *url;
 
-	if (camel_stream_buffer_gets ((CamelStreamBuffer *)http->read, buffer, sizeof (buffer)) <= 0)
-		return -1;
-
-	d(printf("HTTP Status: %s\n", buffer));
-
-	/* parse the HTTP status code */
-	if (!g_ascii_strncasecmp (buffer, "HTTP/", 5)) {
-		token = http_next_token ((const guchar *) buffer);
-		http->statuscode = camel_header_decode_int (&token);
-		return http->statuscode;
+	switch (http->method) {
+	case CAMEL_HTTP_METHOD_GET:
+		method = "GET";
+		break;
+	case CAMEL_HTTP_METHOD_HEAD:
+		method = "HEAD";
+		break;
+	default:
+		g_assert_not_reached ();
 	}
 
-	http_disconnect(http);
+	url = camel_url_to_string (http->url, 0);
 
-	return -1;
+	if (http->proxy) {
+		use_url = url;
+	} else if (http->url->host && *http->url->host) {
+		use_url = strstr (url, http->url->host) + strlen (http->url->host);
+	} else {
+		use_url = http->url->path;
+	}
+
+	d(printf("HTTP Stream Sending: %s %s HTTP/1.0\r\nUser-Agent: %s\r\nHost: %s\r\n",
+		 method,
+		 use_url,
+		 http->user_agent ? http->user_agent : "CamelHttpStream/1.0",
+		 http->url->host));
+	if (camel_stream_printf (
+		http->raw,
+		"%s %s HTTP/1.0\r\nUser-Agent: %s\r\nHost: %s\r\n",
+		method, use_url, http->user_agent ? http->user_agent :
+		"CamelHttpStream/1.0", http->url->host) == -1) {
+		g_set_error (
+			error, G_IO_ERROR,
+			g_io_error_from_errno (errno),
+			"%s", g_strerror (errno));
+		http_disconnect(http);
+		g_free (url);
+		return -1;
+	}
+	g_free (url);
+
+	if (http->authrealm) {
+		d(printf("HTTP Stream Sending: WWW-Authenticate: %s\n", http->authrealm));
+	}
+
+	if (http->authrealm && camel_stream_printf (
+		http->raw, "WWW-Authenticate: %s\r\n",
+		http->authrealm) == -1) {
+		g_set_error (
+			error, G_IO_ERROR,
+			g_io_error_from_errno (errno),
+			"%s", g_strerror (errno));
+		http_disconnect(http);
+		return -1;
+	}
+
+	if (http->authpass && http->proxy) {
+		d(printf("HTTP Stream Sending: Proxy-Aurhorization: Basic %s\n", http->authpass));
+	}
+
+	if (http->authpass && http->proxy && camel_stream_printf (
+		http->raw, "Proxy-Authorization: Basic %s\r\n",
+		http->authpass) == -1) {
+		g_set_error (
+			error, G_IO_ERROR,
+			g_io_error_from_errno (errno),
+			"%s", g_strerror (errno));
+		http_disconnect(http);
+		return -1;
+	}
+
+	/* end the headers */
+	if (camel_stream_write (http->raw, "\r\n", 2, error) == -1 ||
+		camel_stream_flush (http->raw, error) == -1) {
+		http_disconnect(http);
+		return -1;
+	}
+
+	return 0;
 }
 
 static gint
-http_get_headers (CamelHttpStream *http)
+http_get_headers (CamelHttpStream *http,
+                  GError **error)
 {
 	struct _camel_header_raw *headers, *node, *tail;
 	const gchar *type;
@@ -288,10 +222,10 @@ http_get_headers (CamelHttpStream *http)
 	gint err;
 
 	if (http->parser)
-		camel_object_unref (http->parser);
+		g_object_unref (http->parser);
 
 	http->parser = camel_mime_parser_new ();
-	camel_mime_parser_init_with_stream (http->parser, http->read);
+	camel_mime_parser_init_with_stream (http->parser, http->read, NULL);
 
 	switch (camel_mime_parser_step (http->parser, &buf, &len)) {
 	case CAMEL_MIME_PARSER_STATE_MESSAGE:
@@ -332,7 +266,11 @@ http_get_headers (CamelHttpStream *http)
 	err = camel_mime_parser_errno (http->parser);
 
 	if (err != 0) {
-		camel_object_unref (http->parser);
+		g_set_error (
+			error, G_IO_ERROR,
+			g_io_error_from_errno (err),
+			"%s", g_strerror (err));
+		g_object_unref (http->parser);
 		http->parser = NULL;
 		goto exception;
 	}
@@ -347,106 +285,139 @@ http_get_headers (CamelHttpStream *http)
 	return -1;
 }
 
-static gint
-http_method_invoke (CamelHttpStream *http)
+static const gchar *
+http_next_token (const guchar *in)
 {
-	const gchar *method = NULL, *use_url;
-	gchar *url;
+	const guchar *inptr = in;
 
-	switch (http->method) {
-	case CAMEL_HTTP_METHOD_GET:
-		method = "GET";
-		break;
-	case CAMEL_HTTP_METHOD_HEAD:
-		method = "HEAD";
-		break;
-	default:
-		g_assert_not_reached ();
-	}
+	while (*inptr && !isspace ((gint) *inptr))
+		inptr++;
 
-	url = camel_url_to_string (http->url, 0);
+	while (*inptr && isspace ((gint) *inptr))
+		inptr++;
 
-	if (http->proxy) {
-		use_url = url;
-	} else if (http->url->host && *http->url->host) {
-		use_url = strstr (url, http->url->host) + strlen (http->url->host);
-	} else {
-		use_url = http->url->path;
-	}
+	return (const gchar *) inptr;
+}
 
-	d(printf("HTTP Stream Sending: %s %s HTTP/1.0\r\nUser-Agent: %s\r\nHost: %s\r\n",
-		 method,
-		 use_url,
-		 http->user_agent ? http->user_agent : "CamelHttpStream/1.0",
-		 http->url->host));
-	if (camel_stream_printf (http->raw, "%s %s HTTP/1.0\r\nUser-Agent: %s\r\nHost: %s\r\n",
-				 method,
-				 use_url,
-				 http->user_agent ? http->user_agent : "CamelHttpStream/1.0",
-				 http->url->host) == -1) {
-		http_disconnect(http);
-		g_free (url);
+static gint
+http_get_statuscode (CamelHttpStream *http,
+                     GError **error)
+{
+	const gchar *token;
+	gchar buffer[4096];
+
+	if (camel_stream_buffer_gets (
+		CAMEL_STREAM_BUFFER (http->read),
+		buffer, sizeof (buffer), error) <= 0)
 		return -1;
-	}
-	g_free (url);
 
-	if (http->authrealm) {
-		d(printf("HTTP Stream Sending: WWW-Authenticate: %s\n", http->authrealm));
+	d(printf("HTTP Status: %s\n", buffer));
+
+	/* parse the HTTP status code */
+	if (!g_ascii_strncasecmp (buffer, "HTTP/", 5)) {
+		token = http_next_token ((const guchar *) buffer);
+		http->statuscode = camel_header_decode_int (&token);
+		return http->statuscode;
 	}
 
-	if (http->authrealm && camel_stream_printf (http->raw, "WWW-Authenticate: %s\r\n", http->authrealm) == -1) {
-		http_disconnect(http);
-		return -1;
+	http_disconnect(http);
+
+	return -1;
+}
+
+static void
+http_stream_dispose (GObject *object)
+{
+	CamelHttpStream *http = CAMEL_HTTP_STREAM (object);
+
+	if (http->parser != NULL) {
+		g_object_unref (http->parser);
+		http->parser = NULL;
 	}
 
-	if (http->authpass && http->proxy) {
-		d(printf("HTTP Stream Sending: Proxy-Aurhorization: Basic %s\n", http->authpass));
+	if (http->content_type != NULL) {
+		camel_content_type_unref (http->content_type);
+		http->content_type = NULL;
 	}
 
-	if (http->authpass && http->proxy && camel_stream_printf (http->raw, "Proxy-Authorization: Basic %s\r\n",
-								  http->authpass) == -1) {
-		http_disconnect(http);
-		return -1;
+	if (http->session != NULL) {
+		g_object_unref (http->session);
+		http->session = NULL;
 	}
 
-	/* end the headers */
-	if (camel_stream_write (http->raw, "\r\n", 2) == -1 || camel_stream_flush (http->raw) == -1) {
-		http_disconnect(http);
-		return -1;
+	if (http->raw != NULL) {
+		g_object_unref (http->raw);
+		http->raw = NULL;
 	}
 
-	return 0;
+	if (http->read != NULL) {
+		g_object_unref (http->read);
+		http->read = NULL;
+	}
+
+	/* Chain up to parent's dispose() method. */
+	G_OBJECT_CLASS (camel_http_stream_parent_class)->dispose (object);
+}
+
+static void
+http_stream_finalize (GObject *object)
+{
+	CamelHttpStream *http = CAMEL_HTTP_STREAM (object);
+
+	camel_header_raw_clear (&http->headers);
+
+	if (http->url != NULL)
+		camel_url_free (http->url);
+
+	if (http->proxy)
+		camel_url_free (http->proxy);
+
+	g_free (http->authrealm);
+	g_free (http->authpass);
+
+	/* Chain up to parent's finalize() method. */
+	G_OBJECT_CLASS (camel_http_stream_parent_class)->finalize (object);
 }
 
 static gssize
-stream_read (CamelStream *stream, gchar *buffer, gsize n)
+http_stream_read (CamelStream *stream,
+                  gchar *buffer,
+                  gsize n,
+                  GError **error)
 {
 	CamelHttpStream *http = CAMEL_HTTP_STREAM (stream);
 	const gchar *parser_buf;
 	gssize nread;
 
-	if (http->method != CAMEL_HTTP_METHOD_GET && http->method != CAMEL_HTTP_METHOD_HEAD) {
+	if (http->method != CAMEL_HTTP_METHOD_GET &&
+		http->method != CAMEL_HTTP_METHOD_HEAD) {
 		errno = EIO;
+		g_set_error (
+			error, G_IO_ERROR,
+			g_io_error_from_errno (errno),
+			"%s", g_strerror (errno));
 		return -1;
 	}
 
  redirect:
 
 	if (!http->raw) {
-		if (http_connect (http, http->proxy ? http->proxy : http->url) == NULL)
+		if (http_connect (
+			http, http->proxy ? http->proxy :
+			http->url, error) == NULL)
 			return -1;
 
-		if (http_method_invoke (http) == -1) {
+		if (http_method_invoke (http, error) == -1) {
 			http_disconnect(http);
 			return -1;
 		}
 
-		if (http_get_statuscode (http) == -1) {
+		if (http_get_statuscode (http, error) == -1) {
 			http_disconnect(http);
 			return -1;
 		}
 
-		if (http_get_headers (http) == -1) {
+		if (http_get_headers (http, error) == -1) {
 			http_disconnect(http);
 			return -1;
 		}
@@ -501,7 +472,7 @@ stream_read (CamelStream *stream, gchar *buffer, gsize n)
 	if (n == 0)
 		return 0;
 
-	nread = camel_mime_parser_read (http->parser, &parser_buf, n);
+	nread = camel_mime_parser_read (http->parser, &parser_buf, n, error);
 
 	if (nread > 0)
 		memcpy (buffer, parser_buf, nread);
@@ -512,29 +483,34 @@ stream_read (CamelStream *stream, gchar *buffer, gsize n)
 }
 
 static gssize
-stream_write (CamelStream *stream, const gchar *buffer, gsize n)
+http_stream_write (CamelStream *stream,
+                   const gchar *buffer,
+                   gsize n,
+                   GError **error)
 {
 	return -1;
 }
 
 static gint
-stream_flush (CamelStream *stream)
+http_stream_flush (CamelStream *stream,
+                   GError **error)
 {
 	CamelHttpStream *http = (CamelHttpStream *) stream;
 
 	if (http->raw)
-		return camel_stream_flush (http->raw);
+		return camel_stream_flush (http->raw, error);
 	else
 		return 0;
 }
 
 static gint
-stream_close (CamelStream *stream)
+http_stream_close (CamelStream *stream,
+                   GError **error)
 {
 	CamelHttpStream *http = (CamelHttpStream *) stream;
 
 	if (http->raw) {
-		if (camel_stream_close (http->raw) == -1)
+		if (camel_stream_close (http->raw, error) == -1)
 			return -1;
 
 		http_disconnect(http);
@@ -544,7 +520,8 @@ stream_close (CamelStream *stream)
 }
 
 static gint
-stream_reset (CamelStream *stream)
+http_stream_reset (CamelStream *stream,
+                   GError **error)
 {
 	CamelHttpStream *http = CAMEL_HTTP_STREAM (stream);
 
@@ -554,13 +531,67 @@ stream_reset (CamelStream *stream)
 	return 0;
 }
 
+static void
+camel_http_stream_class_init (CamelHttpStreamClass *class)
+{
+	GObjectClass *object_class;
+	CamelStreamClass *stream_class;
+
+	object_class = G_OBJECT_CLASS (class);
+	object_class->dispose = http_stream_dispose;
+	object_class->finalize = http_stream_finalize;
+
+	stream_class = CAMEL_STREAM_CLASS (class);
+	stream_class->read = http_stream_read;
+	stream_class->write = http_stream_write;
+	stream_class->flush = http_stream_flush;
+	stream_class->close = http_stream_close;
+	stream_class->reset = http_stream_reset;
+}
+
+static void
+camel_http_stream_init (CamelHttpStream *http)
+{
+}
+
+/**
+ * camel_http_stream_new:
+ * @method: HTTP method
+ * @session: active session
+ * @url: URL to act upon
+ *
+ * Returns: a http stream
+ **/
+CamelStream *
+camel_http_stream_new (CamelHttpMethod method, struct _CamelSession *session, CamelURL *url)
+{
+	CamelHttpStream *stream;
+	gchar *str;
+
+	g_return_val_if_fail(CAMEL_IS_SESSION(session), NULL);
+	g_return_val_if_fail(url != NULL, NULL);
+
+	stream = g_object_new (CAMEL_TYPE_HTTP_STREAM, NULL);
+
+	stream->method = method;
+	stream->session = g_object_ref (session);
+
+	str = camel_url_to_string (url, 0);
+	stream->url = camel_url_new (str, NULL);
+	g_free (str);
+
+	return (CamelStream *)stream;
+}
+
 CamelContentType *
 camel_http_stream_get_content_type (CamelHttpStream *http_stream)
 {
 	g_return_val_if_fail (CAMEL_IS_HTTP_STREAM (http_stream), NULL);
 
 	if (!http_stream->content_type && !http_stream->raw) {
-		if (stream_read (CAMEL_STREAM (http_stream), NULL, 0) == -1)
+		CamelStream *stream = CAMEL_STREAM (http_stream);
+
+		if (http_stream_read (stream, NULL, 0, NULL) == -1)
 			return NULL;
 	}
 
