@@ -36,8 +36,8 @@
 typedef struct linger CamelLinger;
 #else
 typedef struct {
-	unsigned short l_onoff;
-	unsigned short l_linger;
+	gushort l_onoff;
+	gushort l_linger;
 } CamelLinger;
 #define socklen_t int
 struct addrinfo;
@@ -125,7 +125,7 @@ struct _CamelTcpStreamClass {
 	CamelStreamClass parent_class;
 
 	gint		(*connect)		(CamelTcpStream *stream,
-						 const char *host, const char *service, gint fallback_port,
+						 const gchar *host, const gchar *service, gint fallback_port,
 						 GError **error);
 	gint		(*getsockopt)		(CamelTcpStream *stream,
 						 CamelSockOptData *data);
@@ -143,7 +143,7 @@ struct _CamelTcpStreamClass {
 
 GType		camel_tcp_stream_get_type	(void);
 gint		camel_tcp_stream_connect	(CamelTcpStream *stream,
-						 const char *host, const char *service, gint fallback_port,
+						 const gchar *host, const gchar *service, gint fallback_port,
 						 GError **error);
 gint		camel_tcp_stream_getsockopt	(CamelTcpStream *stream,
 						 CamelSockOptData *data);
